/* Copyright (C) 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.Set;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;

import org.dive4elements.river.artifacts.model.minfo.SedimentLoadLSData;
import org.dive4elements.river.artifacts.model.minfo.SedimentLoadFraction;

// Base class for SedimantLoad$UNITProcessors
public class SedimentLoadProcessor extends DefaultProcessor
{
    private final static Logger logger =
            Logger.getLogger(SedimentLoadProcessor.class);

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        logger.debug("doOut " + bundle.getFacetName());
        CallContext context = generator.getCallContext();
        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                false, // Handle NaN
                theme);
        Object data = bundle.getData(context);
        String facetName = bundle.getFacetName();
        double [][] points;

        logger.debug("Do out for: " + facetName);
        if (FacetTypes.IS.SEDIMENT_LOAD_M3A(facetName) ||
                FacetTypes.IS.SEDIMENT_LOAD_TA(facetName) ||
                FacetTypes.IS.SEDIMENT_LOAD(facetName)) {
            points = (double[][]) data;
        } else if (FacetTypes.IS.SEDIMENT_LOAD_UNKNOWN(facetName)) {
            SedimentLoadLSData load = (SedimentLoadLSData) data;
            Set<Double> kms = load.getKms();
            points = new double[2][kms.size()];
            int counter = 0;
            for (Double km: kms) {
                SedimentLoadFraction fraction = load.getFraction(km);
                points[0][counter] = km;
                points[1][counter] = fraction.getUnknown();
                counter++;
            }
        } else {
            logger.error("Unknown facet name: " + facetName);
            return;
        }

        StyledSeriesBuilder.addPoints(series, points, false); // Keep NaN

        generator.addAxisSeries(series, axisName, visible);
    }
}

